@@ -528,18 +528,6 @@ reset_progress_dialog (NcbOperation *operation)
 	ncb_progress_dialog_clear (NCB_PROGRESS_DIALOG (operation->priv->progress_dialog));
 }
 
-static void
-do_eject (NcbOperation *operation)
-
-{
-	NautilusBurnDrive *drive;
-
-	drive = ncb_selection_peek_drive (operation->priv->selection);
-	if (drive != NULL) {
-		nautilus_burn_drive_eject (drive);
-	}
-}
-
 static void
 do_make_another (NcbOperation *operation)
 {
@@ -569,12 +557,6 @@ finish_progress_dialog (NcbOperation *operation)
 	drive = ncb_selection_peek_drive (operation->priv->selection);
 
 	ncb_progress_dialog_done (NCB_PROGRESS_DIALOG (operation->priv->progress_dialog));
-
-	/* If target is not a disc then don't offer eject */
-	if (nautilus_burn_drive_get_device (drive) == NULL) {
-		gtk_dialog_set_response_sensitive (GTK_DIALOG (operation->priv->progress_dialog),
-						   GTK_RESPONSE_REJECT, FALSE);
-	}
 }
 
 static gboolean
@@ -1125,6 +1107,18 @@ create_iso_track (const char *filename)
 	return track;
 }
 
+static NautilusBurnRecorderTrack *
+create_cue_track (const char *filename)
+{
+       NautilusBurnRecorderTrack *track;
+
+       track = g_new0 (NautilusBurnRecorderTrack, 1);
+       track->type = NAUTILUS_BURN_RECORDER_TRACK_TYPE_CUE;
+       track->contents.cue.filename = g_strdup (filename);
+
+       return track;
+}
+
 static NautilusBurnRecorderTrack *
 create_track_from_device (NcbOperation *operation,
 			  const char   *source_name,
@@ -1181,14 +1175,9 @@ create_track_from_device (NcbOperation *operation,
 			  operation);
 	ncb_progress_dialog_set_operation_string (NCB_PROGRESS_DIALOG (operation->priv->progress_dialog), _("Creating disc image"));
 
-	if (iso_filename != NULL) {
-		/* Don't warn on low space unless the user selected this file specifically */
-		image_flags |= NAUTILUS_BURN_IMAGE_CREATE_WARN_SPACE;
-	}
-
 	toc_filename = NULL;
 	filename     = NULL;
-	res          = NAUTILUS_BURN_RECORDER_RESULT_RETRY;
+	res          = NAUTILUS_BURN_ISO_RESULT_RETRY;
 	temp_paths   = get_temp_paths ();
 	for (i = 0; temp_paths [i] != NULL; i++) {
 
@@ -1209,10 +1198,15 @@ create_track_from_device (NcbOperation *operation,
 							 &image_type,
 							 &toc_filename,
 							 error);
-		if (res != NAUTILUS_BURN_RECORDER_RESULT_RETRY) {
+		if (res != NAUTILUS_BURN_ISO_RESULT_RETRY) {
 			break;
 		}
 
+		/* if this was the last try then save the error */
+		if (temp_paths [i + 1] != NULL) {
+			g_clear_error (error);
+		}
+
 		/* remove temporary file */
 		if (iso_filename == NULL) {
 			remove_file (filename);
@@ -1228,8 +1222,18 @@ create_track_from_device (NcbOperation *operation,
 		goto done;
 	}
 
-	/* track takes ownership of file */
-	track = create_iso_track (filename);
+	switch (image_type) {
+	case NAUTILUS_BURN_IMAGE_TYPE_BINCUE:
+                track = create_cue_track (toc_filename);
+                break;
+
+	case NAUTILUS_BURN_IMAGE_TYPE_ISO9660:
+		track = create_iso_track (filename);
+                break;
+	default:
+		g_assert_not_reached ();
+                break;
+        }
 
  done:
 
@@ -1245,7 +1249,6 @@ create_track_from_device (NcbOperation *operation,
 
 	g_free (label);
 
-	nautilus_burn_drive_eject (source_drive);
 	g_object_unref (source_drive);
 
 	operation->priv->cancel = CANCEL_NONE;
@@ -1355,10 +1358,6 @@ create_track_from_burn_folder (NcbOperation *operation,
 		/* Don't remove the files */
 		nautilus_burn_iso_graft_free (graft, FALSE);
 	} else {
-		if (iso_filename != NULL) {
-			/* Don't warn on low space unless the user selected this file specifically */
-			image_flags |= NAUTILUS_BURN_IMAGE_CREATE_WARN_SPACE;
-		}
 
 		res = NAUTILUS_BURN_ISO_RESULT_ERROR;
 
@@ -1387,6 +1386,11 @@ create_track_from_burn_folder (NcbOperation *operation,
 				break;
 			}
 
+			/* if this was the last try then save the error */
+			if (temp_paths [i + 1] != NULL) {
+				g_clear_error (error);
+			}
+
 			/* remove temporary file */
 			if (iso_filename == NULL) {
 				remove_file (filename);
@@ -1442,18 +1446,6 @@ ncb_operation_set_selection (NcbOperation *operation,
 	}
 }
 
-static NautilusBurnRecorderTrack *
-create_cue_track (const char *filename)
-{
-	NautilusBurnRecorderTrack *track;
-
-	track = g_new0 (NautilusBurnRecorderTrack, 1);
-	track->type = NAUTILUS_BURN_RECORDER_TRACK_TYPE_CUE;
-	track->contents.cue.filename = g_strdup (filename);
-
-	return track;
-}
-
 static gboolean
 prepare_tracks (NcbOperation *operation)
 {
@@ -1530,11 +1522,14 @@ burn_cd (NcbOperation *operation)
 {
 	int                res;
 	NautilusBurnDrive *drive;
+	NcbSelectionSource source_type;
+	char              *source_name;
 
 	drive = ncb_selection_peek_drive (operation->priv->selection);
 
 	if (! prepare_tracks (operation)) {
 		res = NAUTILUS_BURN_RECORDER_RESULT_CANCEL;
+		g_warning ("Unable to prepare tracks for burning");
 		goto out;
 	}
 
@@ -1542,6 +1537,9 @@ burn_cd (NcbOperation *operation)
 		NautilusBurnRecorderTrack *track;
 		char                      *str;
 
+		/* don't show make another button */
+		ncb_progress_dialog_set_make_another_button_visible (NCB_PROGRESS_DIALOG (operation->priv->progress_dialog), FALSE);
+
 		finish_progress_dialog (operation);
 
 		track = operation->priv->tracks->data;
@@ -1555,6 +1553,21 @@ burn_cd (NcbOperation *operation)
 		goto out;
 	}
 
+	/* special case: when copying a disc eject the source if same as destination */
+	source_name = NULL;
+	ncb_selection_get_source (operation->priv->selection, &source_type, &source_name);
+	if (source_type == NCB_SELECTION_SOURCE_DEVICE) {
+		NautilusBurnDrive *source_drive;
+
+		source_drive = nautilus_burn_drive_monitor_get_drive_for_device (nautilus_burn_get_drive_monitor (),
+										 source_name);
+		if (source_drive != NULL
+		    && nautilus_burn_drive_equal (drive, source_drive)) {
+			nautilus_burn_drive_eject (source_drive);
+			g_object_unref (source_drive);
+		}
+	}
+
 	res = write_disc (operation);
 
  out:
@@ -1706,10 +1719,6 @@ progress_dialog_response (GtkWidget    *dialog,
 		do_make_another (operation);
 	}
 
-	if (response_id == GTK_RESPONSE_REJECT) {
-		do_eject (operation);
-	}
-
 	if (response_id == GTK_RESPONSE_CLOSE) {
 		gtk_widget_hide (dialog);
 		do_callback (operation);
